// EntropyPool.cpp: implementation of the CEntropyPool class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RandGen.h"
#include "EntropyPool.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEntropyPool::CEntropyPool()
{
	m_pData = NULL;
	m_nBufSize = 0;

	Reset();
	Grow(1024);
}

CEntropyPool::~CEntropyPool()
{
	if (m_pData != NULL)
		free(m_pData);
}

void CEntropyPool::Reset()
{
	m_nDataSize = 0;
	m_nEntropyBits = 0;
}

unsigned int CEntropyPool::ApproxateEntropy(unsigned int nValue, unsigned int nMaxBitsEntropy)
{
	unsigned int nTempValue = nValue;
	unsigned int nBitsToRepresentValue = 1;
	unsigned int nApproxEntropy = 0;

	while (nTempValue != 0) {
		nTempValue >>= 1;
		nBitsToRepresentValue++;
	}

	// assume that half of the bits needed to represent the value
	// have entropy.  (This means larger values are assumed to
	// have more entropy than small ones.)  This should be a fairly
	// conservative estimate.
	nApproxEntropy = nBitsToRepresentValue / 2;

	// don't return more than max value
	if (nApproxEntropy > nMaxBitsEntropy)
		nApproxEntropy = nMaxBitsEntropy;

	return nApproxEntropy;
}

void CEntropyPool::AddData(int nData, unsigned int nEntropyBits)
{
	AddData(&nData, sizeof(nData), nEntropyBits);
}

void CEntropyPool::AddData(void *pData, size_t nSize, unsigned int nEntropyBits)
{
	if ((m_nDataSize + nSize) > m_nBufSize) {
		Grow(nSize);
	}

	memcpy(m_pData + m_nDataSize, pData, nSize);
	m_nDataSize += nSize;
	m_nEntropyBits += nEntropyBits;
}

void CEntropyPool::Grow(size_t nGrowBy)
{
	if (nGrowBy < 1024)
		nGrowBy = 1024;

	m_pData = (BYTE *)realloc(m_pData, m_nBufSize + nGrowBy);
	m_nBufSize += nGrowBy;
}
